using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Utils.Logs;
using Utils.Misc;
using WLog.Class;
using Utils.Logs.Event;
using System.Collections;

namespace WLog.GUI
{
    /// <summary>
    /// Delegate for GUI refreshing
    /// </summary>
    internal delegate void RefreshGUIDelegate();
    /// <summary>
    /// Main frame
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// LogSet which contains all the logs
        /// </summary>
        private LogSet vLogSet = new LogSet();
        /// <summary>
        /// Indicates whether the HourComboBox is filling
        /// </summary>
        private bool vFillComboBox = true;
        /// <summary>
        /// Indicates the number of file changes
        /// </summary>
        private int vFileChanges = 0;
        /// <summary>
        /// Delegate for the GUI refreshing
        /// </summary>
        private RefreshGUIDelegate dRefreshGui;
        /// <summary>
        /// Options class
        /// </summary>
        private Options opt;
        /// <summary>
        /// Indicates whether an event changed the CheckBoxes
        /// </summary>
        private bool vEventCheck = false;
        /// <summary>
        /// Path for the log file
        /// </summary>
        private string vLogPath = "";
        /// <summary>
        /// File changed EventHandler
        /// </summary>
        private FileChangedEventHandler vFileChangedEventHandler;
        /// <summary>
        /// Indicates the index of the marked row
        /// </summary>
        private int vMarkedRowIndex = -1;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            vFileChangedEventHandler = new FileChangedEventHandler(Event_FileChanged);
            tscb_Hours.SelectedIndexChanged += new EventHandler(Event_Hours_SelectedIndexChanged);
            dRefreshGui = new RefreshGUIDelegate(RefreshGUI);
            if (!OptionContainer.Instance.Affected)
                OptionContainer.Instance.Options = new Options();
            opt = (Options)OptionContainer.Instance.Options;
            opt.OptionChanged += new EventHandler(Event_OptionChanged);
            SetOptions();
        }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_LogFile">Path of the Log file</param>
        public fr_Main(string _LogFile)
        {
            InitializeComponent();
            vFileChangedEventHandler = new FileChangedEventHandler(Event_FileChanged);
            tscb_Hours.SelectedIndexChanged += new EventHandler(Event_Hours_SelectedIndexChanged);
            dRefreshGui = new RefreshGUIDelegate(RefreshGUI);
            if (!OptionContainer.Instance.Affected)
                OptionContainer.Instance.Options = new Options();
            opt = (Options)OptionContainer.Instance.Options;
            opt.OptionChanged += new EventHandler(Event_OptionChanged);
            SetOptions();
            vLogPath = _LogFile;
        }
        #endregion

        #region Events
        /// <summary>
        /// Frame load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
#if DEBUG
            this.tsmi_MenuDebug.Visible = true;
#endif
            if (vLogPath != "")
            {
                this.OpenLog(vLogPath);
                SetOptions();
            }
        }
        /// <summary>
        /// Displays the LogSet data with a ListView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ListViewMode(object sender, EventArgs e)
        {
            fr_ListView fr = new fr_ListView();
            //fr.LogSet = this.vLogSet;
            fr.ShowDialog();
        }
        /// <summary>
        /// Open log file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_OpenLog(object sender, EventArgs e)
        {
            DialogResult dr = ofd_OpenLog.ShowDialog();
            if (dr != DialogResult.OK) return;
            this.vLogPath = ofd_OpenLog.FileName;

            OpenLog(this.vLogPath);
        }
        /// <summary>
        /// Close the logs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_CloseLog(object sender, EventArgs e)
        {
            this.tsMenu_Mark.Enabled = false;
            this.vLogSet = new LogSet();
            this.dgv_Log.DataSource = null;
            this.tsmi_Search.Enabled = false;
            this.gb_Menu.Enabled = false;
            this.tsMenu_Open.Enabled = true;
            this.tsMenu_Close.Enabled = false;
            this.vFillComboBox = true;
            this.tssl_LogsNum.Text = "Entries: 0";
            this.tssl_Size.Text = "File size : 0 Ko";
            this.tssl_FileChanged.Text = "File changes: 0";
            this.tssl_Guard.Text = "Guard mode: OFF";
            this.tsMenu_Guard.Enabled = false;
            this.tssl_Guard.Enabled = false;

            if (tsMenu_Guard.Checked)
            {
                vLogSet.StopGuard();
                tsMenu_Guard.Checked = false;
            }
        }
        /// <summary>
        /// Search log by type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SearchByType(object sender, EventArgs e)
        {
            if (!vEventCheck)
            {
                vEventCheck = true;
                List<LogType> ltList = new List<LogType>();

                if (tsMenu_Debug.Checked) ltList.Add(LogType.Debug);
                if (tsMenu_Error.Checked) ltList.Add(LogType.Error);
                if (tsMenu_Information.Checked) ltList.Add(LogType.Information);
                if (tsMenu_Miscellaneous.Checked) ltList.Add(LogType.Miscellaneous);
                if (tsMenu_Report.Checked) ltList.Add(LogType.Report);

                #region Type searching
                if (sender == tsMenu_All)
                {
                    #region IsMenuVisible & CheckBoxes
                    tsMenu_Debug.Checked = true;
                    tsMenu_Error.Checked = true;
                    tsMenu_Information.Checked = true;
                    tsMenu_Miscellaneous.Checked = true;
                    tsMenu_Report.Checked = true;

                    cb_Debug.Checked = true;
                    cb_Error.Checked = true;
                    cb_Information.Checked = true;
                    cb_Miscellaneous.Checked = true;
                    cb_Report.Checked = true;
                    #endregion

                    if (!ltList.Contains(LogType.Debug))
                        ltList.Add(LogType.Debug);
                    if (!ltList.Contains(LogType.Miscellaneous))
                        ltList.Add(LogType.Miscellaneous);
                    if (!ltList.Contains(LogType.Error))
                        ltList.Add(LogType.Error);
                    if (!ltList.Contains(LogType.Information))
                        ltList.Add(LogType.Information);
                    if (!ltList.Contains(LogType.Report))
                        ltList.Add(LogType.Report);
                }
                else if (sender == tsMenu_None)
                {
                    #region IsMenuVisible & CheckBoxes
                    tsMenu_Debug.Checked = false;
                    tsMenu_Error.Checked = false;
                    tsMenu_Information.Checked = false;
                    tsMenu_Miscellaneous.Checked = false;
                    tsMenu_Report.Checked = false;

                    cb_Debug.Checked = false;
                    cb_Error.Checked = false;
                    cb_Information.Checked = false;
                    cb_Miscellaneous.Checked = false;
                    cb_Report.Checked = false;
                    #endregion

                    ltList.Clear();
                }
                else if (sender == tsMenu_Debug || sender == cb_Debug)
                {
                    if (!tsMenu_Debug.Checked)
                    {
                        tsMenu_Debug.Checked = true;
                        cb_Debug.Checked = true;
                        if (!ltList.Contains(LogType.Debug))
                            ltList.Add(LogType.Debug);
                    }
                    else
                    {
                        tsMenu_Debug.Checked = false;
                        cb_Debug.Checked = false;
                        ltList.Remove(LogType.Debug);
                    }
                }
                else if (sender == tsMenu_Error || sender == cb_Error)
                {
                    if (!tsMenu_Error.Checked)
                    {
                        tsMenu_Error.Checked = true;
                        cb_Error.Checked = true;
                        if (!ltList.Contains(LogType.Error))
                            ltList.Add(LogType.Error);
                    }
                    else
                    {
                        tsMenu_Error.Checked = false;
                        cb_Error.Checked = false;
                        ltList.Remove(LogType.Error);
                    }
                }
                else if (sender == tsMenu_Information || sender == cb_Information)
                {
                    if (!tsMenu_Information.Checked)
                    {
                        tsMenu_Information.Checked = true;
                        cb_Information.Checked = true;
                        if (!ltList.Contains(LogType.Information))
                            ltList.Add(LogType.Information);
                    }
                    else
                    {
                        tsMenu_Information.Checked = false;
                        cb_Information.Checked = false;
                        ltList.Remove(LogType.Information);
                    }
                }

                else if (sender == tsMenu_Miscellaneous || sender == cb_Miscellaneous)
                {
                    if (!tsMenu_Miscellaneous.Checked)
                    {
                        tsMenu_Miscellaneous.Checked = true;
                        cb_Miscellaneous.Checked = true;
                        if (!ltList.Contains(LogType.Miscellaneous))
                            ltList.Add(LogType.Miscellaneous);
                    }
                    else
                    {
                        tsMenu_Miscellaneous.Checked = false;
                        cb_Miscellaneous.Checked = false;
                        ltList.Remove(LogType.Miscellaneous);
                    }
                }

                else if (sender == tsMenu_Report || sender == cb_Report)
                {
                    if (!tsMenu_Report.Checked)
                    {
                        tsMenu_Report.Checked = true;
                        cb_Report.Checked = true;
                        if (!ltList.Contains(LogType.Report))
                            ltList.Add(LogType.Report);
                    }
                    else
                    {
                        tsMenu_Report.Checked = false;
                        cb_Report.Checked = false;
                        ltList.Remove(LogType.Report);
                    }
                }
                #endregion

                dgv_Log.DataSource = null;
                LogSet _Result = new LogSet(this.vLogSet.FilePath);
                _Result = this.vLogSet.Search(ltList);

                _Result.List.Sort(new LogRowIDComparer());
                dgv_Log.DataSource = _Result;
                this.RefreshHourComboBox();
                vEventCheck = false;
                this.RefreshGUI();
            }
        }
        /// <summary>
        /// Click on options
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            Components.GUI.fr_OptionBox fr = new Components.GUI.fr_OptionBox();
            fr.OptionsObject = OptionContainer.Instance.Options;
            fr.ShowDialog();
        }
        /// <summary>
        /// About box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_About(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Indicates whether WLog will listen for change in the log file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_GuardFile(object sender, EventArgs e)
        {
            if (!tsMenu_Guard.Checked)
            {
                tsMenu_Guard.Checked = true;
                this.vLogSet.StartGuard();
                this.tssl_Guard.Text = "Guard mode: ON";
                vLogSet.FileChanged += vFileChangedEventHandler;
            }
            else
            {
                tsMenu_Guard.Checked = false;
                this.vLogSet.StopGuard();
                this.tssl_Guard.Text = "Guard mode: OFF";
                this.vLogSet.FileChanged -= vFileChangedEventHandler;
            }
        }
        /// <summary>
        /// Quit the application
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Quit(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Ask for the calendar view
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_CalView(object sender, EventArgs e)
        {
            try
            {
                fr_Cal fr = new fr_Cal();
                LogMeetingSet lms = new LogMeetingSet();
                lms.LogPath = this.vLogPath;
                lms.Fill();
                fr.MeetingSource = lms;
                fr.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible to load the calendar (" + ex.Message
                    + ")", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Add row in the file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Menuclick_AddRow(object sender, EventArgs e)
        {
            Log.Instance.Write("NEw row added in file", "Add", LogType.Information);
        }
        /// <summary>
        /// Mark the current row in color
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Mark(object sender, EventArgs e)
        {
            if (vMarkedRowIndex == -1)
            {
                DataGridViewCellStyle style = new DataGridViewCellStyle();
                style.BackColor = Color.Blue;
                style.ForeColor = Color.White;
                dgv_Log.SelectedRows[0].DefaultCellStyle = style;
                vMarkedRowIndex = dgv_Log.SelectedRows[0].Index;
                dgv_Log.SelectedRows[0].Selected = false;
                tsMenu_Mark.Text = "Unselect the marked row.";
            }
            else
            {
                SetRowColor(dgv_Log.Rows[vMarkedRowIndex]);
                vMarkedRowIndex = -1;
                tsMenu_Mark.Text = "Mark selected row.";
            }
        }
        /// <summary>
        /// The selected hour changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Hours_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!vFillComboBox)
            {
                LogSet _Result = new LogSet(this.vLogSet.FilePath);
                DateTime dt = DateTime.Parse((string)tscb_Hours.SelectedItem);
                _Result = this.vLogSet.Search(dt);
                _Result.List.Sort(new LogRowIDComparer());
                dgv_Log.DataSource = _Result;
            }
            else
                vFillComboBox = false;
        }
        /// <summary>
        /// Indicates that the file was modified
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FileChanged(object sender, FileChangedEventArgs e)
        {
            this.Invoke(this.dRefreshGui);
        }
        /// <summary>
        /// Double click in the DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_DGV_DoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            lock (this)
            {
                fr_LogReader fr = new fr_LogReader();
                fr.LogSet = this.vLogSet;
                fr.Index = dgv_Log.SelectedRows[0].Index;
                dgv_Log.SelectedRows[0].Selected = false;
                fr.ShowDialog();
            }
        }
        /// <summary>
        /// Options changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            SetOptions();
        }
        /// <summary>
        /// Click on All or None
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_AllNone(object sender, EventArgs e)
        {
            if (sender == btn_All)
            {
                vEventCheck = true;
                cb_Debug.Checked = true; tsMenu_Debug.Checked = true;
                cb_Error.Checked = true; tsMenu_Error.Checked = true;
                cb_Information.Checked = true; tsMenu_Information.Checked = true;
                cb_Miscellaneous.Checked = true; tsMenu_Miscellaneous.Checked = true;
                vEventCheck = false;
                cb_Report.Checked = true; tsMenu_Report.Checked = true;
            }
            if (sender == btn_None)
            {
                vEventCheck = true;
                cb_Debug.Checked = false; tsMenu_Debug.Checked = false;
                cb_Error.Checked = false; tsMenu_Error.Checked = false;
                cb_Information.Checked = false; tsMenu_Information.Checked = false;
                cb_Miscellaneous.Checked = false; tsMenu_Miscellaneous.Checked = false;
                vEventCheck = false;
                cb_Report.Checked = false; tsMenu_Report.Checked = false;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Refresh the GUI
        /// </summary>
        private void RefreshGUI()
        {
            tssl_FileChanged.Text = "File changes: " + (++vFileChanges);
            try
            {
                this.dgv_Log.DataSource = null;
                this.vLogSet.Fill();

                this.vFillComboBox = true;
                this.dgv_Log.DataSource = this.vLogSet.List;
                this.RefreshLabels(this.vLogSet);
                dgv_Log.Rows[dgv_Log.SelectedRows[0].Index].Selected = false;
                int sbOffset = dgv_Log.VerticalScrollingOffset;

                SetDataGridView();
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, this, LogType.Error);
            }
        }
        /// <summary>
        /// Refreshes the hours ComboBoxes
        /// </summary>
        private void RefreshHourComboBox()
        {
            this.vFillComboBox = true;
            this.tscb_Hours.Items.Clear();
            foreach (DateTime dt in vLogSet.ListOfDates)
            {
                this.tscb_Hours.ComboBox.Items.Add(dt.ToShortDateString());
            }
            if (this.tscb_Hours.ComboBox.Items.Count > 0)
                this.tscb_Hours.ComboBox.SelectedIndex = 0;
            this.vFillComboBox = false;
        }
        /// <summary>
        /// Refresh the labels
        /// </summary>
        private void RefreshLabels(LogSet _LogSet)
        {
            this.tssl_LogsNum.Text = "Entries: " + _LogSet.Count;
            this.tssl_Size.Text = "File size : " + _LogSet.FileSize.ToString("0.00") + " Ko";
        }
        /// <summary>
        /// Sets the options
        /// </summary>
        private void SetOptions()
        {
            #region Set DGV columns
            try
            {
                RefreshGUI();
            }
            catch (Exception) { }
            #endregion

            #region IsMenuVisible
            if (!opt.IsMenuVisible)
                tlp_General.RowStyles[0].Height = 0;
            else
                tlp_General.RowStyles[0].Height = Options.SIZE_OF_MENU;
            #endregion
        }
        /// <summary>
        /// Open logs and displays it
        /// </summary>
        /// <param name="_FilePath">Path of the log file</param>
        private void OpenLog(string _FilePath)
        {
            try
            {
                this.dgv_Log.DataSource = null;
                this.vLogSet.FilePath = _FilePath;
                this.vLogSet.Fill();

                int RowsCount = vLogSet.Count;
                this.vFillComboBox = true;
                this.dgv_Log.DataSource = this.vLogSet;
                this.RefreshLabels(this.vLogSet);

                RefreshHourComboBox();

                this.tsmi_Search.Enabled = true;
                this.gb_Menu.Enabled = true;
                this.tsMenu_Open.Enabled = false;
                this.tsMenu_Close.Enabled = true;
                this.tsMenu_Guard.Enabled = true;
                this.tssl_Guard.Enabled = true;
                this.tsMenu_Mark.Enabled = true;
                this.vFileChanges = 0;
                this.SetDataGridView();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible to fill the logs (" + ex.Message + ")", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Set the options of the DataGridView
        /// </summary>
        private void SetDataGridView()
        {
            try
            {
                Options opt = (Options)OptionContainer.Instance.Options;

                #region Display types
                List<LogType> ltList = new List<LogType>();

                if (tsMenu_Debug.Checked) ltList.Add(LogType.Debug);
                if (tsMenu_Error.Checked) ltList.Add(LogType.Error);
                if (tsMenu_Information.Checked) ltList.Add(LogType.Information);
                if (tsMenu_Miscellaneous.Checked) ltList.Add(LogType.Miscellaneous);
                if (tsMenu_Report.Checked) ltList.Add(LogType.Report);
                dgv_Log.DataSource = null;
                LogSet _Result = new LogSet(this.vLogSet.FilePath);
                _Result = this.vLogSet.Search(ltList);

                _Result.List.Sort(new LogRowIDComparer());
                dgv_Log.DataSource = _Result;

                this.RefreshHourComboBox();
                vEventCheck = false;
                #endregion

                if (dgv_Log.Rows.Count <= 0) return;
                dgv_Log.FirstDisplayedScrollingRowIndex = dgv_Log.Rows.Count - 1;

                dgv_Log.SelectedRows[0].Selected = false;

                dgv_Log.Columns["Date"].Visible = false;
                dgv_Log.Columns["ID"].Visible = false;
                dgv_Log.Columns["ParsedDate"].Visible = opt.IsDateVisible;
                dgv_Log.Columns["ParsedDate"].HeaderText = "Date";
                dgv_Log.Columns["ParsedDate"].DisplayIndex = 0;

                dgv_Log.Columns["Type"].Visible = opt.IsTypeLogVisible;
                dgv_Log.Columns["Type"].DisplayIndex = 1;

                dgv_Log.Columns["Title"].Visible = opt.IsTitleVisible;
                dgv_Log.Columns["Title"].DisplayIndex = 2;

                dgv_Log.Columns["Message"].Visible = opt.IsMessageVisible;
                dgv_Log.Columns["Message"].DisplayIndex = 3;

                #region Color of the rows
                if (opt.IsColored)
                {
                    for (int i = 0; i < dgv_Log.Rows.Count; i++)
                    {
                        SetRowColor(dgv_Log.Rows[i]);
                    }
                }
                #endregion

                #region Select the marked row
                if (vMarkedRowIndex != -1 && vMarkedRowIndex < dgv_Log.Rows.Count)
                {
                    DataGridViewCellStyle style = new DataGridViewCellStyle();
                    style.BackColor = Color.Blue;
                    style.ForeColor = Color.White;
                    dgv_Log.Rows[vMarkedRowIndex].DefaultCellStyle = style;
                }
                #endregion
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Set the color of the row
        /// </summary>
        /// <param name="_Row">The row to color</param>
        private void SetRowColor(DataGridViewRow _Row)
        {
            try
            {
                Options opt = (Options)OptionContainer.Instance.Options;
                if (!opt.IsTitleVisible) return;
                DataGridViewCellStyle style = _Row.DefaultCellStyle;
                if ((LogType)_Row.Cells["Type"].Value == LogType.Debug)
                {
                    style.BackColor = Color.Yellow;
                    style.ForeColor = Color.Black;
                }
                else if ((LogType)_Row.Cells["Type"].Value == LogType.Error)
                {
                    style.BackColor = Color.Red;
                    style.ForeColor = Color.White;
                }
                else if ((LogType)_Row.Cells["Type"].Value == LogType.Information)
                {
                    style.BackColor = Color.White;
                    style.ForeColor = Color.Black;
                }
                else if ((LogType)_Row.Cells["Type"].Value == LogType.Miscellaneous)
                {
                    style.BackColor = Color.LightBlue;
                    style.ForeColor = Color.Black;
                }
                else if ((LogType)_Row.Cells["Type"].Value == LogType.Report)
                {
                    style.BackColor = Color.LightGreen;
                    style.ForeColor = Color.Black;
                }
                _Row.DefaultCellStyle = style;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        #endregion
    }
}